@@ -1,7 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import os
-import xml.dom.minidom
 from difflib import SequenceMatcher
 from collections import defaultdict
 from hashlib import md5
@@ -15,6 +14,7 @@ from sql.conditionals import Case
 from sql.operators import Or, And
 from sql.aggregate import Max
 
+from defusedxml.minidom import parseString
 from genshi.filters.i18n import extract as genshi_extract
 from relatorio.reporting import MIMETemplateLoader
 from relatorio.templates.opendocument import get_zip_file
@@ -177,7 +177,7 @@ class Translation(ModelSQL, ModelView):
                     trans_help[trans['name']] = trans
                 elif trans['type'] == 'selection':
                     trans_selection.setdefault(trans['name'], {})
-                    trans_selection[trans['name']][trans['src']] = trans
+                    trans_selection[trans['name']][trans['src']] = trans or ''
 
         def update_insert_field(field, trans_name):
             string_md5 = cls.get_src_md5(field.string)
@@ -219,7 +219,7 @@ class Translation(ModelSQL, ModelView):
         def insert_selection(field, trans_name):
             for (_, val) in field.selection:
                 if (trans_name not in trans_selection
-                        or val not in trans_selection[trans_name]):
+                        or (val or '') not in trans_selection[trans_name]):
                     val_md5 = cls.get_src_md5(val)
                     cursor.execute(*ir_translation.insert(
                             [ir_translation.name, ir_translation.lang,
@@ -970,7 +970,7 @@ class TranslationSet(Wizard):
                 zip_.read('content.xml'),
                 zip_.read('styles.xml'),
                 ]:
-            document = xml.dom.minidom.parseString(content_xml)
+            document = parseString(content_xml)
             for string in extract(document.documentElement):
                 yield string
 
