@@ -1,7 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-from trytond.model import ModelView, ModelSQL, fields
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from collections import defaultdict
+from itertools import groupby
+
+from trytond.model import (
+    ModelView, ModelSQL, DeactivableMixin, fields, sequence_ordered, tree)
 from trytond.transaction import Transaction
+from trytond.tools import grouped_slice
 from trytond.pool import Pool
 from trytond.rpc import RPC
 
@@ -18,61 +23,68 @@ def one_in(i, j):
             return True
     return False
 
-CLIENT_ICONS = [(x, x) for x in (
-    'tryton-attachment-hi',
-    'tryton-attachment',
-    'tryton-cancel',
-    'tryton-clear',
-    'tryton-close',
-    'tryton-connect',
-    'tryton-copy',
-    'tryton-delete',
-    'tryton-dialog-error',
-    'tryton-dialog-information',
-    'tryton-dialog-warning',
-    'tryton-disconnect',
-    'tryton-executable',
-    'tryton-find-replace',
-    'tryton-find',
-    'tryton-folder-new',
-    'tryton-fullscreen',
-    'tryton-go-home',
-    'tryton-go-jump',
-    'tryton-go-next',
-    'tryton-go-previous',
-    'tryton-help',
-    'tryton-icon',
-    'tryton-list-add',
-    'tryton-list-remove',
-    'tryton-locale',
-    'tryton-lock',
-    'tryton-log-out',
-    'tryton-mail-message-new',
-    'tryton-mail-message',
-    'tryton-new',
-    'tryton-ok',
-    'tryton-open',
-    'tryton-preferences-system-session',
-    'tryton-preferences-system',
-    'tryton-preferences',
-    'tryton-print',
-    'tryton-refresh',
-    'tryton-save-as',
-    'tryton-save',
-    'tryton-start-here',
-    'tryton-system-file-manager',
-    'tryton-system',
-    'tryton-undo',
-    'tryton-web-browser')]
-SEPARATOR = ' / '
+
+CLIENT_ICONS = [(x, x) for x in [
+        'tryton-add',
+        'tryton-archive',
+        'tryton-attach',
+        'tryton-back',
+        'tryton-bookmark-border',
+        'tryton-bookmark',
+        'tryton-bookmarks',
+        'tryton-cancel',
+        'tryton-clear',
+        'tryton-close',
+        'tryton-copy',
+        'tryton-create',
+        'tryton-date',
+        'tryton-delete',
+        'tryton-email',
+        'tryton-error',
+        'tryton-exit',
+        'tryton-export',
+        'tryton-filter',
+        'tryton-format-align-center',
+        'tryton-format-align-justify',
+        'tryton-format-align-left',
+        'tryton-format-align-right',
+        'tryton-format-bold',
+        'tryton-format-color-text',
+        'tryton-format-italic',
+        'tryton-format-underline',
+        'tryton-forward',
+        'tryton-history',
+        'tryton-import',
+        'tryton-info',
+        'tryton-launch',
+        'tryton-link',
+        'tryton-log',
+        'tryton-menu',
+        'tryton-note',
+        'tryton-ok',
+        'tryton-open',
+        'tryton-print',
+        'tryton-public',
+        'tryton-refresh',
+        'tryton-remove',
+        'tryton-save',
+        'tryton-search',
+        'tryton-star-border',
+        'tryton-star',
+        'tryton-switch',
+        'tryton-translate',
+        'tryton-unarchive',
+        'tryton-undo',
+        'tryton-warning',
+        ]]
 
 
-class UIMenu(ModelSQL, ModelView):
+class UIMenu(DeactivableMixin, sequence_ordered(), tree(separator=' / '),
+        ModelSQL, ModelView):
     "UI menu"
     __name__ = 'ir.ui.menu'
 
     name = fields.Char('Menu', required=True, translate=True)
-    sequence = fields.Integer('Sequence', required=True)
     childs = fields.One2Many('ir.ui.menu', 'parent', 'Children')
     parent = fields.Many2One('ir.ui.menu', 'Parent Menu', select=True,
             ondelete='CASCADE')
@@ -89,34 +101,22 @@ class UIMenu(ModelSQL, ModelView):
                 ('ir.action.wizard', 'ir.action.wizard'),
                 ('ir.action.url', 'ir.action.url'),
                 ]), 'get_action', setter='set_action')
-    active = fields.Boolean('Active')
+    action_keywords = fields.One2Many('ir.action.keyword', 'model',
+        'Action Keywords')
     favorite = fields.Function(fields.Boolean('Favorite'), 'get_favorite')
 
     @classmethod
-    def __setup__(cls):
-        super(UIMenu, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
-        cls._error_messages.update({
-                'wrong_name': ('"%%s" is not a valid menu name because it is '
-                    'not allowed to contain "%s".' % SEPARATOR),
-                })
-
-    @classmethod
-    def order_complet_name(cls, tables):
+    def order_complete_name(cls, tables):
         return cls.name.convert_order('name', tables, cls)
 
     @staticmethod
     def default_icon():
-        return 'tryton-open'
+        return 'tryton-folder'
 
     @staticmethod
     def default_sequence():
         return 10
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def list_icons():
         pool = Pool()
@@ -124,40 +124,6 @@ class UIMenu(ModelSQL, ModelView):
         return sorted(CLIENT_ICONS
             + [(name, name) for _, name in Icon.list_icons()])
 
-    @classmethod
-    def validate(cls, menus):
-        super(UIMenu, cls).validate(menus)
-        cls.check_recursion(menus)
-        for menu in menus:
-            menu.check_name()
-
-    def check_name(self):
-        if SEPARATOR in self.name:
-            self.raise_user_error('wrong_name', (self.name,))
-
-    def get_rec_name(self, name):
-        parent = self.parent
-        name = self.name
-        while parent:
-            name = parent.name + SEPARATOR + name
-            parent = parent.parent
-        return name
-
-    @classmethod
-    def search_rec_name(cls, name, clause):
-        if isinstance(clause[2], basestring):
-            values = clause[2].split(SEPARATOR.strip())
-            values.reverse()
-            domain = []
-            field = 'name'
-            for name in values:
-                domain.append((field, clause[1], name.strip()))
-                field = 'parent.' + field
-        else:
-            domain = [('name',) + tuple(clause[1:])]
-        ids = [m.id for m in cls.search(domain, order=[])]
-        return [('parent', 'child_of', ids)]
-
     @classmethod
     def search_global(cls, text):
         # TODO improve search clause
@@ -165,7 +131,7 @@ class UIMenu(ModelSQL, ModelView):
                     ('rec_name', 'ilike', '%%%s%%' % text),
                     ]):
             if record.action:
-                yield record.id, record.rec_name, record.icon
+                yield record, record.rec_name, record.icon
 
     @classmethod
     def search(cls, domain, offset=0, limit=None, order=None, count=False,
@@ -176,12 +142,15 @@ class UIMenu(ModelSQL, ModelView):
             return menus
 
         if menus:
-            parent_ids = [x.parent.id for x in menus if x.parent]
-            parents = cls.search([
-                    ('id', 'in', parent_ids),
-                    ])
-            menus = [x for x in menus
-                if (x.parent and x.parent in parents) or not x.parent]
+            parent_ids = {x.parent.id for x in menus if x.parent}
+            parents = set()
+            for sub_parent_ids in grouped_slice(parent_ids):
+                parents.update(cls.search([
+                            ('id', 'in', list(sub_parent_ids)),
+                            ]))
+            # Re-browse to avoid side-cache access
+            menus = cls.browse([x.id for x in menus
+                    if (x.parent and x.parent in parents) or not x.parent])
 
         if count:
             return len(menus)
@@ -190,37 +159,39 @@ class UIMenu(ModelSQL, ModelView):
     @classmethod
     def get_action(cls, menus, name):
         pool = Pool()
-        ActionKeyword = pool.get('ir.action.keyword')
         actions = dict((m.id, None) for m in menus)
         with Transaction().set_context(active_test=False):
-            action_keywords = ActionKeyword.search([
-                    ('keyword', '=', 'tree_open'),
-                    ('model', 'in', [str(m) for m in menus]),
-                    ])
-        for action_keyword in action_keywords:
-            model = action_keyword.model
-            Action = pool.get(action_keyword.action.type)
+            menus = cls.browse(menus)
+        action_keywords = sum((list(m.action_keywords) for m in menus), [])
+        key = lambda k: k.action.type
+        action_keywords.sort(key=key)
+        for type, action_keywords in groupby(action_keywords, key=key):
+            action_keywords = list(action_keywords)
+            action2keywords = defaultdict(list)
+            for action_keyword in action_keywords:
+                model = action_keyword.model
+                actions[model.id] = '%s,-1' % type
+                action2keywords[action_keyword.action.id].append(
+                    action_keyword)
+
+            Action = pool.get(type)
             with Transaction().set_context(active_test=False):
                 factions = Action.search([
-                        ('action', '=', action_keyword.action.id),
-                        ], limit=1)
-            if factions:
-                action, = factions
-            else:
-                action = '%s,0' % action_keyword.action.type
-            actions[model.id] = str(action)
+                        ('action', 'in', list(action2keywords.keys())),
+                        ])
+            for action in factions:
+                for action_keyword in action2keywords[action.id]:
+                    actions[action_keyword.model.id] = str(action)
         return actions
 
     @classmethod
     def set_action(cls, menus, name, value):
-        if not value:
-            return
         pool = Pool()
         ActionKeyword = pool.get('ir.action.keyword')
         action_keywords = []
-        cursor = Transaction().cursor
-        for i in range(0, len(menus), cursor.IN_MAX):
-            sub_menus = menus[i:i + cursor.IN_MAX]
+        transaction = Transaction()
+        for i in range(0, len(menus), transaction.database.IN_MAX):
+            sub_menus = menus[i:i + transaction.database.IN_MAX]
             action_keywords += ActionKeyword.search([
                 ('keyword', '=', 'tree_open'),
                 ('model', 'in', [str(menu) for menu in sub_menus]),
@@ -228,11 +199,13 @@ class UIMenu(ModelSQL, ModelView):
         if action_keywords:
             with Transaction().set_context(_timestamp=False):
                 ActionKeyword.delete(action_keywords)
-        if isinstance(value, basestring):
+        if not value:
+            return
+        if isinstance(value, str):
             action_type, action_id = value.split(',')
         else:
             action_type, action_id = value
-        if not int(action_id):
+        if int(action_id) <= 0:
             return
         Action = pool.get(action_type)
         action = Action(int(action_id))
@@ -262,13 +235,12 @@ class UIMenu(ModelSQL, ModelView):
         return menu2favorite
 
 
-class UIMenuFavorite(ModelSQL, ModelView):
+class UIMenuFavorite(sequence_ordered(), ModelSQL, ModelView):
     "Menu Favorite"
     __name__ = 'ir.ui.menu.favorite'
 
     menu = fields.Many2One('ir.ui.menu', 'Menu', required=True,
         ondelete='CASCADE')
-    sequence = fields.Integer('Sequence')
     user = fields.Many2One('res.user', 'User', required=True,
         ondelete='CASCADE')
 
@@ -280,15 +252,6 @@ class UIMenuFavorite(ModelSQL, ModelView):
                 'set': RPC(readonly=False),
                 'unset': RPC(readonly=False),
                 })
-        cls._order = [
-            ('sequence', 'ASC'),
-            ('id', 'DESC'),
-            ]
-
-    @staticmethod
-    def order_sequence(tables):
-        table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
 
     @staticmethod
     def default_user():
