@@ -1,13 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 import copy
-import collections
-import warnings
+from functools import total_ordering
 
 from trytond.model import fields
 from trytond.error import WarningErrorMixin
-from trytond.pool import Pool, PoolBase
+from trytond.pool import Pool, PoolBase, PoolMeta
 from trytond.pyson import PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.url import URLMixin
@@ -16,7 +15,16 @@ from trytond.rpc import RPC
 __all__ = ['Model']
 
 
-class Model(WarningErrorMixin, URLMixin, PoolBase):
+class ModelMeta(PoolMeta):
+    @property
+    def __queue__(self):
+        pool = Pool()
+        Queue = pool.get('ir.queue')
+        return Queue.caller(self)
+
+
+@total_ordering
+class Model(WarningErrorMixin, URLMixin, PoolBase, metaclass=ModelMeta):
     """
     Define a model in Tryton.
     """
@@ -30,16 +38,10 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         cls.__rpc__ = {
             'default_get': RPC(),
             'fields_get': RPC(),
-            'on_change_with': RPC(instantiate=0),
             'pre_validate': RPC(instantiate=0),
             }
         cls._error_messages = {}
 
-        if hasattr(cls, '__depend_methods'):
-            cls.__depend_methods = cls.__depend_methods.copy()
-        else:
-            cls.__depend_methods = collections.defaultdict(set)
-
         # Copy fields and update depends
         for attr in dir(cls):
             if attr.startswith('_'):
@@ -47,28 +49,16 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
             if not isinstance(getattr(cls, attr), fields.Field):
                 continue
             field_name = attr
-            field = copy.deepcopy(getattr(cls, field_name))
+            field = getattr(cls, field_name)
+            # Copy the original field definition to prevent side-effect with
+            # the mutable attributes
+            for parent_cls in cls.__mro__:
+                parent_field = getattr(parent_cls, field_name, None)
+                if isinstance(parent_field, fields.Field):
+                    field = parent_field
+            field = copy.deepcopy(field)
             setattr(cls, field_name, field)
 
-            for attribute in ('on_change', 'on_change_with', 'autocomplete',
-                    'selection_change_with'):
-                if attribute == 'selection_change_with':
-                    if isinstance(
-                            getattr(field, 'selection', None), basestring):
-                        function_name = field.selection
-                    else:
-                        continue
-                else:
-                    function_name = '%s_%s' % (attribute, field_name)
-                function = getattr(cls, function_name, None)
-                if function:
-                    if getattr(function, 'depends', None):
-                        setattr(field, attribute,
-                            getattr(field, attribute) | function.depends)
-                    if getattr(function, 'depend_methods', None):
-                        cls.__depend_methods[(field_name, attribute)] |= \
-                            function.depend_methods
-
     @classmethod
     def __post_setup__(cls):
         super(Model, cls).__post_setup__()
@@ -83,10 +73,10 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
 
         # Set _defaults
         cls._defaults = {}
-        fields_names = cls._fields.keys()
+        fields_names = list(cls._fields.keys())
         for field_name in fields_names:
             default_method = getattr(cls, 'default_%s' % field_name, False)
-            if isinstance(default_method, collections.Callable):
+            if callable(default_method):
                 cls._defaults[field_name] = default_method
 
         for k in cls._defaults:
@@ -94,32 +84,8 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                 'Default function defined in %s but field %s does not exist!' \
                 % (cls.__name__, k,)
 
-        # Update __rpc__
-        for field_name, field in cls._fields.iteritems():
-            if isinstance(field, (fields.Selection, fields.Reference)) \
-                    and not isinstance(field.selection, (list, tuple)) \
-                    and field.selection not in cls.__rpc__:
-                instantiate = 0 if field.selection_change_with else None
-                cls.__rpc__.setdefault(field.selection,
-                    RPC(instantiate=instantiate))
-
-            for attribute in ('on_change', 'on_change_with', 'autocomplete'):
-                function_name = '%s_%s' % (attribute, field_name)
-                if getattr(cls, function_name, None):
-                    cls.__rpc__.setdefault(function_name, RPC(instantiate=0))
-
-        # Update depend on methods
-        for (field_name, attribute), others in (
-                cls.__depend_methods.iteritems()):
-            field = getattr(cls, field_name)
-            for other in others:
-                other_field = getattr(cls, other)
-                setattr(field, attribute,
-                    getattr(field, attribute)
-                    | getattr(other_field, attribute))
-
         # Set name to fields
-        for name, field in cls._fields.iteritems():
+        for name, field in cls._fields.items():
             if field.name is None:
                 field.name = name
             else:
@@ -132,6 +98,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         '''
         Returns the first non-empty line of the model docstring.
         '''
+        assert cls.__doc__, '%s has no docstring' % cls
         lines = cls.__doc__.splitlines()
         for line in lines:
             line = line.strip()
@@ -157,27 +124,28 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         Translation.register_error_messages(cls, module_name)
 
     @classmethod
-    def default_get(cls, fields_names, with_rec_name=True,
-            with_on_change=True):
+    def default_get(cls, fields_names, with_rec_name=True):
         '''
         Return a dict with the default values for each field in fields_names.
         If with_rec_name is True, rec_name will be added.
-        If with_on_change is True, on_change will be added.
         '''
         pool = Pool()
-        Property = pool.get('ir.property')
         value = {}
 
+        default_rec_name = Transaction().context.get('default_rec_name')
+        if (default_rec_name
+                and cls._rec_name in cls._fields
+                and cls._rec_name in fields_names):
+            value[cls._rec_name] = default_rec_name
+
         # get the default values defined in the object
         for field_name in fields_names:
             if field_name in cls._defaults:
                 value[field_name] = cls._defaults[field_name]()
             field = cls._fields[field_name]
             if (field._type == 'boolean'
-                    and not field_name in value):
+                    and field_name not in value):
                 value[field_name] = False
-            if isinstance(field, fields.Property):
-                value[field_name] = Property.get(field_name, cls.__name__)
             if (with_rec_name
                     and field._type in ('many2one',)
                     and value.get(field_name)):
@@ -186,37 +154,12 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                     value[field_name + '.rec_name'] = Target(
                         value[field_name]).rec_name
 
-        if with_on_change:
-            value = cls._default_on_change(value)
         if not with_rec_name:
-            for field in value.keys():
+            for field in list(value.keys()):
                 if field.endswith('.rec_name'):
                     del value[field]
         return value
 
-    @classmethod
-    def _default_on_change(cls, value):
-        """
-        Call on_change function for the default value
-        and return new default value
-        """
-        pool = Pool()
-        res = value.copy()
-        val = {}
-        for field in value.keys():
-            if field in cls._fields:
-                if cls._fields[field].on_change:
-                    inst = cls()
-                    for fname in cls._fields[field].on_change:
-                        setattr(inst, fname, value.get(fname))
-                    val.update(getattr(inst, 'on_change_' + field)())
-                if cls._fields[field]._type in ('one2many',):
-                    Target = pool.get(cls._fields[field].model_name)
-                    for val2 in res[field]:
-                        val2.update(Target._default_on_change(val2))
-        res.update(val)
-        return res
-
     @classmethod
     def fields_get(cls, fields_names=None):
         """
@@ -228,7 +171,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         FieldAccess = pool.get('ir.model.field.access')
         ModelAccess = pool.get('ir.model.access')
 
-        #Add translation to cache
+        # Add translation to cache
         language = Transaction().language
         trans_args = []
         for field in (x for x in cls._fields.keys()
@@ -278,6 +221,9 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                     'filename',
                     'selection_change_with',
                     'domain',
+                    'converter',
+                    'search_order',
+                    'search_context',
                     ):
                 if getattr(cls._fields[field], arg, None) is not None:
                     value = getattr(cls._fields[field], arg)
@@ -296,13 +242,17 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                         and getattr(cls._fields[field], arg):
                     res[field][arg] = copy.copy(getattr(cls._fields[field],
                         arg))
-            if (isinstance(cls._fields[field],
-                        (fields.Function, fields.One2Many, fields.Many2Many))
-                    and not getattr(cls, 'order_%s' % field, None)):
+            if ((isinstance(cls._fields[field],
+                            (fields.Function, fields.One2Many,
+                                fields.Many2Many))
+                        and not getattr(cls, 'order_%s' % field, None))
+                    or not hasattr(cls, 'search')):
                 res[field]['sortable'] = False
             if ((isinstance(cls._fields[field], fields.Function)
-                    and not cls._fields[field].searcher)
-                    or cls._fields[field]._type in ('binary', 'sha')):
+                        and not (cls._fields[field].searcher
+                            or getattr(cls, 'domain_%s' % field, None)))
+                    or (cls._fields[field]._type in ('binary', 'sha'))
+                    or not hasattr(cls, 'search')):
                 res[field]['searchable'] = False
             else:
                 res[field]['searchable'] = True
@@ -365,13 +315,16 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                         cls._fields[field].field)
             if res[field]['type'] == 'many2one':
                 target = cls._fields[field].get_target()
-                for target_name, target_field in target._fields.iteritems():
+                relation_fields = []
+                for target_name, target_field in target._fields.items():
                     if (target_field._type == 'one2many'
                             and target_field.model_name == cls.__name__
                             and target_field.field == field):
-                        res[field]['relation_field'] = target_name
-                        break
-            if res[field]['type'] in ('datetime', 'time'):
+                        relation_fields.append(target_name)
+                # Set relation_field only if there is no ambiguity
+                if len(relation_fields) == 1:
+                    res[field]['relation_field'], = relation_fields
+            if res[field]['type'] in ('datetime', 'time', 'timestamp'):
                 res[field]['format'] = copy.copy(cls._fields[field].format)
             if res[field]['type'] == 'selection':
                 res[field]['context'] = copy.copy(cls._fields[field].context)
@@ -383,14 +336,17 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                     True)
                 res[field]['delete'] = accesses.get(field, {}).get('delete',
                     True)
+            filter_ = getattr(cls._fields[field], 'filter', None)
+            if filter_:
+                res[field]['domain'] = ['AND', res[field]['domain'], filter_]
 
             # convert attributes into pyson
             for attr in ('states', 'domain', 'context', 'digits', 'size',
-                    'add_remove', 'format'):
+                    'add_remove', 'format', 'search_order', 'search_context'):
                 if attr in res[field]:
                     res[field][attr] = encoder.encode(res[field][attr])
 
-        for i in res.keys():
+        for i in list(res.keys()):
             # filter out fields which aren't in the fields_names list
             if fields_names:
                 if i not in fields_names:
@@ -399,13 +355,6 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                 del res[i]
         return res
 
-    def on_change_with(self, fieldnames):
-        changes = {}
-        for fieldname in fieldnames:
-            method_name = 'on_change_with_%s' % fieldname
-            changes[fieldname] = getattr(self, method_name)()
-        return changes
-
     def pre_validate(self):
         pass
 
@@ -413,41 +362,41 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         super(Model, self).__init__()
         if id is not None:
             id = int(id)
-        self.__dict__['id'] = id
-        self._values = None
-        parent_values = {}
-        for name, value in kwargs.iteritems():
-            if not name.startswith('_parent_'):
-                setattr(self, name, value)
-            else:
-                parent_values[name] = value
-        for name, value in parent_values.iteritems():
-            parent_name, field = name.split('.', 1)
-            parent_name = parent_name[8:]  # Strip '_parent_'
-            parent = getattr(self, parent_name, None)
-            if parent is not None:
-                setattr(parent, field, value)
-            else:
-                setattr(self, parent_name, {field: value})
+        self._id = id
+        if kwargs:
+            self._values = {}
+            parent_values = {}
+            for name, value in kwargs.items():
+                if not name.startswith('_parent_'):
+                    setattr(self, name, value)
+                else:
+                    parent_values[name] = value
+
+            def set_parent_value(record, name, value):
+                parent_name, field = name.split('.', 1)
+                parent_name = parent_name[8:]  # Strip '_parent_'
+                parent = getattr(record, parent_name, None)
+                if parent is not None:
+                    if not field.startswith('_parent_'):
+                        setattr(parent, field, value)
+                    else:
+                        set_parent_value(parent, field, value)
+                else:
+                    setattr(record, parent_name, {field: value})
+
+            for name, value in parent_values.items():
+                set_parent_value(self, name, value)
+            self._init_values = self._values.copy()
+        else:
+            self._values = None
+            self._init_values = None
 
     def __getattr__(self, name):
-        if name == 'id':
-            return self.__dict__['id']
-        elif self._values and name in self._values:
-            return self._values.get(name)
-        raise AttributeError("'%s' Model has no attribute '%s': %s"
-            % (self.__name__, name, self._values))
-
-    def __setattr__(self, name, value):
-        if name == 'id':
-            self.__dict__['id'] = value
-            return
-        super(Model, self).__setattr__(name, value)
-
-    def __getitem__(self, name):
-        warnings.warn('Use __getattr__ instead of __getitem__',
-            DeprecationWarning, stacklevel=2)
-        return getattr(self, name)
+        try:
+            return self._values[name]
+        except (KeyError, TypeError):
+            raise AttributeError("'%s' Model has no attribute '%s': %s"
+                % (self.__name__, name, self._values))
 
     def __contains__(self, name):
         return name in self._fields
@@ -458,11 +407,8 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
     def __str__(self):
         return '%s,%s' % (self.__name__, self.id)
 
-    def __unicode__(self):
-        return u'%s,%s' % (self.__name__, self.id)
-
     def __repr__(self):
-        if self.id < 0:
+        if self.id is None or self.id < 0:
             return "Pool().get('%s')(**%s)" % (self.__name__,
                 repr(self._default_values))
         else:
@@ -471,8 +417,8 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
     def __eq__(self, other):
         if not isinstance(other, Model):
             return NotImplemented
-        if self.id is None or other.id is None:
-            return False
+        elif self.id is None or other.id is None:
+            return id(self) == id(other)
         return (self.__name__, self.id) == (other.__name__, other.id)
 
     def __lt__(self, other):
@@ -480,38 +426,38 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
             return NotImplemented
         return self.id < other.id
 
-    # TODO: replace by total_ordering when 2.6 will be dropped
-    __gt__ = lambda self, other: not (self < other or self == other)
-    __le__ = lambda self, other: self < other or self == other
-    __ge__ = lambda self, other: not self < other
-
     def __ne__(self, other):
-        if not isinstance(other, Model):
-            return NotImplemented
-        if self.id is None or other.id is None:
-            return True
-        return (self.__name__, self.id) != (other.__name__, other.id)
+        return not self == other
 
     def __hash__(self):
         return hash((self.__name__, self.id))
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
 
     @property
     def _default_values(self):
-        if self.id >= 0:
-            return self.id
+        """Return the values not stored.
+        By default, the value of a field is its internal representation except:
+            - for Many2One and One2One field: the id
+            - for Reference field: the string model,id
+            - for Many2Many: the list of ids
+            - for One2Many: the list of `_default_values`
+        """
         values = {}
         if self._values:
-            for fname, value in self._values.iteritems():
+            for fname, value in self._values.items():
                 field = self._fields[fname]
-                if isinstance(field, fields.Reference):
-                    if value is not None:
-                        value = str(value)
-                elif isinstance(value, Model):
-                    value = value._default_values
-                elif isinstance(value, list):
-                    value = [r._default_values for r in value]
+                if field._type in ('many2one', 'one2one', 'reference'):
+                    if value:
+                        if field._type == 'reference':
+                            value = str(value)
+                        else:
+                            value = value.id
+                elif field._type in ('one2many', 'many2many'):
+                    if field._type == 'one2many':
+                        value = [r._default_values for r in value]
+                    else:
+                        value = [r.id for r in value]
                 values[fname] = value
         return values
