@@ -1,17 +1,15 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-from ..model import ModelSQL, fields
-from ..backend import TableHandler
-from ..transaction import Transaction
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from ..model import ModelSQL, DeactivableMixin, fields
 from ..pool import Pool, PoolMeta
 
 __all__ = [
     'UIMenuGroup', 'ActionGroup', 'ModelFieldGroup', 'ModelButtonGroup',
-    'RuleGroupGroup', 'RuleGroupUser', 'Lang', 'SequenceType',
+    'ModelButtonRule', 'ModelButtonClick',
+    'RuleGroupGroup', 'Lang', 'SequenceType',
     'SequenceTypeGroup', 'Sequence', 'SequenceStrict',
     'ModuleConfigWizardItem',
     ]
-__metaclass__ = PoolMeta
 
 
 class UIMenuGroup(ModelSQL):
@@ -23,28 +21,15 @@ class UIMenuGroup(ModelSQL):
             select=True, required=True)
 
     @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'ir_ui_menu_group_rel', cls._table)
-        TableHandler.sequence_rename(cursor, 'ir_ui_menu_group_rel_id_seq',
-                cls._table + '_id_seq')
-        # Migration from 2.0 menu_id and gid renamed into menu group
-        table = TableHandler(cursor, cls, module_name)
-        table.column_rename('menu_id', 'menu')
-        table.column_rename('gid', 'group')
-        super(UIMenuGroup, cls).__register__(module_name)
-
-    @classmethod
-    def create(cls, vals):
-        res = super(UIMenuGroup, cls).create(vals)
+    def create(cls, vlist):
+        res = super(UIMenuGroup, cls).create(vlist)
         # Restart the cache on the domain_get method
         Pool().get('ir.rule')._domain_get_cache.clear()
         return res
 
     @classmethod
-    def write(cls, records, vals):
-        super(UIMenuGroup, cls).write(records, vals)
+    def write(cls, records, values, *args):
+        super(UIMenuGroup, cls).write(records, values, *args)
         # Restart the cache on the domain_get method
         Pool().get('ir.rule')._domain_get_cache.clear()
 
@@ -64,36 +49,28 @@ class ActionGroup(ModelSQL):
             select=True, required=True)
 
     @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'ir_action_group_rel', cls._table)
-        TableHandler.sequence_rename(cursor, 'ir_action_group_rel_id_seq',
-                cls._table + '_id_seq')
-        # Migration from 2.0 action_id and gid renamed into action and group
-        table = TableHandler(cursor, cls, module_name)
-        table.column_rename('action_id', 'action')
-        table.column_rename('gid', 'group')
-        super(ActionGroup, cls).__register__(module_name)
-
-    @classmethod
-    def create(cls, vals):
+    def create(cls, vlist):
         Action = Pool().get('ir.action')
-        if vals.get('action'):
-            vals = vals.copy()
-            vals['action'] = Action.get_action_id(vals['action'])
-        res = super(ActionGroup, cls).create(vals)
+        vlist = [x.copy() for x in vlist]
+        for vals in vlist:
+            if vals.get('action'):
+                vals['action'] = Action.get_action_id(vals['action'])
+        res = super(ActionGroup, cls).create(vlist)
         # Restart the cache on the domain_get method
         Pool().get('ir.rule')._domain_get_cache.clear()
         return res
 
     @classmethod
-    def write(cls, records, vals):
+    def write(cls, records, values, *args):
         Action = Pool().get('ir.action')
-        if vals.get('action'):
-            vals = vals.copy()
-            vals['action'] = Action.get_action_id(vals['action'])
-        super(ActionGroup, cls).write(records, vals)
+        actions = iter((records, values) + args)
+        args = []
+        for records, values in zip(actions, actions):
+            if values.get('action'):
+                values = values.copy()
+                values['action'] = Action.get_action_id(values['action'])
+            args.extend((records, values))
+        super(ActionGroup, cls).write(*args)
         # Restart the cache on the domain_get method
         Pool().get('ir.rule')._domain_get_cache.clear()
 
@@ -107,47 +84,32 @@ class ActionGroup(ModelSQL):
 class ModelFieldGroup(ModelSQL):
     "Model Field Group Rel"
     __name__ = 'ir.model.field-res.group'
-    field_id = fields.Many2One('ir.model.field', 'Model Field',
+    field = fields.Many2One('ir.model.field', 'Model Field',
             ondelete='CASCADE', select=True, required=True)
-    group_id = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
+    group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'ir_model_field_group_rel',
-            cls._table)
-        TableHandler.sequence_rename(cursor, 'ir_model_field_group_rel_id_seq',
-            cls._table + '_id_seq')
-        super(ModelFieldGroup, cls).__register__(module_name)
 
-
-class ModelButtonGroup(ModelSQL):
+class ModelButtonGroup(DeactivableMixin, ModelSQL):
     "Model Button - Group"
     __name__ = 'ir.model.button-res.group'
     button = fields.Many2One('ir.model.button', 'Button',
         ondelete='CASCADE', select=True, required=True)
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
         select=True, required=True)
-    active = fields.Boolean('Active', select=True)
-
-    @staticmethod
-    def default_active():
-        return True
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         pool = Pool()
-        result = super(ModelButtonGroup, cls).create(values)
+        result = super(ModelButtonGroup, cls).create(vlist)
         # Restart the cache for get_groups
         pool.get('ir.model.button')._groups_cache.clear()
         return result
 
     @classmethod
-    def write(cls, records, values):
+    def write(cls, records, values, *args):
         pool = Pool()
-        super(ModelButtonGroup, cls).write(records, values)
+        super(ModelButtonGroup, cls).write(records, values, *args)
         # Restart the cache for get_groups
         pool.get('ir.model.button')._groups_cache.clear()
 
@@ -159,6 +121,16 @@ class ModelButtonGroup(ModelSQL):
         pool.get('ir.model.button')._groups_cache.clear()
 
 
+class ModelButtonRule(metaclass=PoolMeta):
+    __name__ = 'ir.model.button.rule'
+    group = fields.Many2One('res.group', "Group", ondelete='CASCADE')
+
+
+class ModelButtonClick(metaclass=PoolMeta):
+    __name__ = 'ir.model.button.click'
+    user = fields.Many2One('res.user', "User", ondelete='CASCADE')
+
+
 class RuleGroupGroup(ModelSQL):
     "Rule Group - Group"
     __name__ = 'ir.rule.group-res.group'
@@ -167,55 +139,18 @@ class RuleGroupGroup(ModelSQL):
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'group_rule_group_rel', cls._table)
-        TableHandler.sequence_rename(cursor, 'group_rule_group_rel_id_seq',
-                cls._table + '_id_seq')
-        # Migration from 2.0 rule_group_id and group_id renamed into rule_group
-        # and group
-        table = TableHandler(cursor, cls, module_name)
-        table.column_rename('rule_group_id', 'rule_group')
-        table.column_rename('group_id', 'group')
-        super(RuleGroupGroup, cls).__register__(module_name)
 
-
-class RuleGroupUser(ModelSQL):
-    "Rule Group - User"
-    __name__ = 'ir.rule.group-res.user'
-    rule_group = fields.Many2One('ir.rule.group', 'Rule Group',
-            ondelete='CASCADE', select=True, required=True)
-    user = fields.Many2One('res.user', 'User', ondelete='CASCADE',
-            select=True, required=True)
-
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'user_rule_group_rel', cls._table)
-        TableHandler.sequence_rename(cursor, 'user_rule_group_rel_id_seq',
-                cls._table + '_id_seq')
-        # Migration from 2.0 rule_group_id and user_id renamed into rule_group
-        # and user
-        table = TableHandler(cursor, cls, module_name)
-        table.column_rename('rule_group_id', 'rule_group')
-        table.column_rename('user_id', 'user')
-        super(RuleGroupUser, cls).__register__(module_name)
-
-
-class Lang:
+class Lang(metaclass=PoolMeta):
     __name__ = 'ir.lang'
 
     @classmethod
-    def write(cls, langs, vals):
-        super(Lang, cls).write(langs, vals)
+    def write(cls, langs, values, *args):
+        super(Lang, cls).write(langs, values, *args)
         # Restart the cache for get_preferences
         Pool().get('res.user')._get_preferences_cache.clear()
 
 
-class SequenceType:
+class SequenceType(metaclass=PoolMeta):
     __name__ = 'ir.sequence.type'
     groups = fields.Many2Many('ir.sequence.type-res.group', 'sequence_type',
             'group', 'User Groups',
@@ -238,22 +173,22 @@ class SequenceTypeGroup(ModelSQL):
         Rule._domain_get_cache.clear()
 
     @classmethod
-    def create(cls, vals):
+    def create(cls, vlist):
         Rule = Pool().get('ir.rule')
-        res = super(SequenceTypeGroup, cls).create(vals)
+        res = super(SequenceTypeGroup, cls).create(vlist)
         # Restart the cache on the domain_get method of ir.rule
         Rule._domain_get_cache.clear()
         return res
 
     @classmethod
-    def write(cls, records, vals):
+    def write(cls, records, values, *args):
         Rule = Pool().get('ir.rule')
-        super(SequenceTypeGroup, cls).write(records, vals)
+        super(SequenceTypeGroup, cls).write(records, values, *args)
         # Restart the cache on the domain_get method
         Rule._domain_get_cache.clear()
 
 
-class Sequence:
+class Sequence(metaclass=PoolMeta):
     __name__ = 'ir.sequence'
     groups = fields.Function(fields.Many2Many('res.group', None, None,
         'User Groups'), 'get_groups', searcher='search_groups')
@@ -266,7 +201,7 @@ class Sequence:
             code2seq.setdefault(sequence.code, []).append(sequence.id)
 
         sequence_types = SequenceType.search([
-                ('code', 'in', code2seq.keys()),
+                ('code', 'in', list(code2seq.keys())),
                 ])
 
         groups = {}
@@ -291,23 +226,23 @@ class SequenceStrict(Sequence):
     __name__ = 'ir.sequence.strict'
 
 
-class ModuleConfigWizardItem:
-    __name__ = 'ir.module.module.config_wizard.item'
+class ModuleConfigWizardItem(metaclass=PoolMeta):
+    __name__ = 'ir.module.config_wizard.item'
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         pool = Pool()
         User = pool.get('res.user')
-        result = super(ModuleConfigWizardItem, cls).create(values)
+        result = super(ModuleConfigWizardItem, cls).create(vlist)
         # Restart the cache for get_preferences
         User._get_preferences_cache.clear()
         return result
 
     @classmethod
-    def write(cls, items, values):
+    def write(cls, items, values, *args):
         pool = Pool()
         User = pool.get('res.user')
-        super(ModuleConfigWizardItem, cls).write(items, values)
+        super(ModuleConfigWizardItem, cls).write(items, values, *args)
         # Restart the cache for get_preferences
         User._get_preferences_cache.clear()
 
