@@ -9,6 +9,7 @@ import subprocess
 import sys
 import time
 import unittest
+import unittest.mock
 from functools import reduce
 from functools import wraps
 from itertools import chain
@@ -673,6 +674,7 @@ def doctest_setup(test):
 
 
 def doctest_teardown(test):
+    unittest.mock.patch.stopall()
     return drop_db()
 
 
