@@ -35,8 +35,9 @@ class BaseCache(object):
 
     def _key(self, key):
         if self.context:
-            context = Transaction().context
+            context = Transaction().context.copy()
             context.pop('client', None)
+            context.pop('_request', None)
             return (key, Transaction().user, freeze(context))
         return key
 
